#####  GAME ESTIMATION EXAMPLE  #####
#
# AMPL Code for Estimating Price Games with Multiple Equilibrium
# 
# Constrained Optimization Approaches to Estimation of 
# Structural Models
#
# Ken Judd and Che-Lin Su (April 2007)
#
#####  SET UP THE PROBLEM  #####

# param sigma := 3;
# param m := 1;
# param A := 50;
param nCITY := 4;
param nDataPts := 10000;

# param gamma := 1/2;

set CITY := 1..4;
set DATA := 1..nDataPts;

param N := 30;
set P := 1..N; 	# P is the index set of prices
#  p[i] denotes i-th price; the set of prices is a uniform grid on the interval [pmin, pmax]
param pmin := 1;
param pmax := 30;
param p {i in P} := pmin + (pmax-pmin)/(N-1)*(i-1); 

param n {i in CITY} ;
# := 
# 1   1500
# 2   2500
# 3   3000
# 4   4000 ;

param pxSOL {i in CITY} ;
# :=
# 1  24.2391 
# 2  25.1804
# 3  8.452
# 4  1.71165;

param pySOL {i in CITY} ;
# :=
# 1  24.2391 
# 2  2.18692
# 3  23.9631
# 4  1.71165;

option randseed 100;

param PDATAx {i in CITY, j in DATA} := 50 * Normal01() + pxSOL[i];
param PDATAy {i in CITY, j in DATA} := 50 * Normal01() + pySOL[i];

param AVGpx {i in CITY} := sum {j in DATA} PDATAx[i, j] / nDataPts;
param AVGpy {i in CITY} := sum {j in DATA} PDATAy[i, j] / nDataPts;

var Ax >= 0; 
var Ay >= 0;

var mx >= 0;
var my >= 0;

var px {i in CITY} ;
var py {i in CITY} ;

var gamma;
var sigma;

var Dx1 {i in CITY} = Ax - px[i];
var Dx2 {i in CITY};
var Dy2 {i in CITY};
var Dy3 {i in CITY} = Ay - py[i];

# Given py[i] fixed, Dx2p[i,j] is the demand of the good x from the second type customers in city i with price p[j]
var Dx2p {i in CITY, j in P} = n[i]*p[j]^(-sigma)*(p[j]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma))) ; 

# Given px[i] fixed, Dy2p[i,j] is the demand of the good y from the second type customers in city i with price p[j]
var Dy2p {i in CITY, j in P} = n[i]*p[j]^(-sigma)*(px[i]^(1-sigma)+p[j]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma))) ;

var GlobalConstrProfitDiffX {i in CITY, j in P} = (px[i]-mx) * (Dx1[i]+ Dx2[i]) - (p[j]-mx) * (Ax - p[j] + Dx2p[i,j]) ; 

var GlobalConstrProfitDiffY {i in CITY, j in P} = (py[i]-my) * (Dy3[i]+ Dy2[i]) - (p[j]-my) * (Ay - p[j] + Dy2p[i,j]) ; 


## # Likelihood function 
maximize Likelihood: - sum {i in CITY, j in DATA} (PDATAx[i,j] - px[i])^2 -  sum {i in CITY, j in DATA} (PDATAy[i,j] - py[i])^2;

### Demand of Type 2 Customer ####

DemandX2 {i in CITY} : Dx2[i] =  n[i]*px[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma))) ; 

DemandY2 {i in CITY}: Dy2[i] =  n[i]*py[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma))) ; 

## FOCs:
mrvx {i in CITY}: Dx1[i] + Dx2[i] + (-mx+px[i])*(-1 - sigma*Dx2[i]/px[i] - Dx2[i]*(1-gamma*sigma)/(px[i]^sigma*gamma*(px[i]^(1-sigma) + py[i]^(1-sigma)))) = 0;
 
mrvy {i in CITY}: Dy3[i] + Dy2[i] + (-my+py[i])*(-1 - sigma*Dy2[i]/py[i] - Dy2[i]*(1-gamma*sigma)/(py[i]^sigma*gamma*(px[i]^(1-sigma) + py[i]^(1-sigma)))) = 0;

## Constraint on A and m:

AxConstr {i in CITY} : 4/5* Ax >= px[i]; 
AyConstr {i in CITY} : 4/5* Ay >= py[i];

mxConstr {i in CITY} : mx <= px[i];
myConstr {i in CITY} : my <= py[i];

AConstr : Ax = Ay;
mConstr : mx = my;

# Constraints for global optimization:

GlobalConstrPX {i in CITY, j in P} : (px[i]-mx) * (Dx1[i]+ Dx2[i]) >= (p[j]-mx) * (Ax - p[j] + Dx2p[i,j]) ; 

GlobalConstrPY {i in CITY, j in P} : (py[i]-my) * (Dy3[i]+ Dy2[i]) >= (p[j]-my) * (Ay - p[j] + Dy2p[i,j]) ; 

####   DEFINE THE PROBLEM   #####
#
# Name the problem
problem PriceGameMLE:
#
# Choose the objective function
Likelihood,

#
# List the variables
px, py, Dx1, Dx2, Dy2, Dy3, gamma, sigma, mx, my, Ax, Ay, Dx2p, Dy2p,

#
# List the constraints
DemandX2,
DemandY2,
mrvx, 
mrvy, 
AxConstr,
AyConstr,
mxConstr,
myConstr,
AConstr,
mConstr, 
GlobalConstrPX,
GlobalConstrPY ;

#
#################################

data;

param n := 
1   1500
2   2500
3   3000
4   4000 ;

param pxSOL :=
1  24.2391 
2  25.1804
3  2.14616
4  1.71165;

param pySOL :=
1  24.2391 
2  2.18692
3  25.1209
4  1.71165;


display AVGpx, AVGpy;


printf " This is the BEGINNING for Case 1: Estimate: sigma, gamma \n \n " > Case1.txt;
printf "%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n", "Round", "sigma", "gamma", "Ax", "Ay", "mx", "my", "px[1]", "py[1]", "px[2]", "py[2]", "px[3]", "py[3]", "px[4]", "py[4]", "Objective Value", "Results" > Case1.txt;

printf " This is the BEGINNING for Case 2: Estimate: sigma, gamma, A, m \n \n " > Case2.txt;
printf "%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n", "Round", "sigma", "gamma",  "Ax", "Ay", "mx", "my", "px[1]", "py[1]", "px[2]", "py[2]", "px[3]", "py[3]", "px[4]", "py[4]", "Objective Value", "Results" > Case2.txt;

printf " This is the BEGINNING for Case 3: Estimate: sigma, gamma, Ax, Ay, mx, my \n \n " > Case3.txt;
printf "%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n", "Round", "sigma", "gamma",  "Ax", "Ay", "mx", "my", "px[1]", "py[1]", "px[2]", "py[2]", "px[3]", "py[3]", "px[4]", "py[4]", "Objective Value", "Results" > Case3.txt;


for {k in 1..25}{

#########################
# Case 1: Estimate: sigma, gamma
#                    
#########################

let {i in CITY} px[i] := AVGpx[i];

let {i in CITY} py[i] := AVGpy[i];

let Ax := 50;
let Ay := 50;
let mx := 1;
let my := 1;


let gamma := 0.25 + (2-0.25)/(25-1)*(k-1);
let sigma := 4 - (4-2)/(25-1)*(k-1);

let {i in CITY} Dx2[i] :=  n[i]*px[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));
let {i in CITY} Dy2[i] :=  n[i]*py[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));

#### Change this to the location of your solver
# option knitro_options 'alg=1 maxit=2000', solver "/data/bin/knitro-ampl";
option knitro_options 'alg=1 maxit=2000', solver "/Users/clsu/KnitroAMPL/knitroampl";

fix Ax;
fix Ay;
fix mx ;
fix my;

solve PriceGameMLE;

printf "%u\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %15.6g\t  ", k, sigma, gamma, Ax, Ay, mx, my, px[1], py[1], px[2], py[2], px[3], py[3], px[4], py[4], Likelihood  > Case1.txt;

if match (solve_message, "LOCALLY OPTIMAL SOLUTION FOUND") > 0 then { 
			printf "%s\n", "Optimal solution found" > Case1.txt;
			}
		else if match (solve_message, "Iteration limit reached") > 0 then { 
			printf "%s\n", "Iteration limit reached" > Case1.txt;
			}
            else {
			printf "%s\n", "No solution" > Case1.txt;
            }


display px, py, gamma, sigma, Ax, Ay, mx, my;
display Dx1, Dx2, Dy2, Dy3;

display GlobalConstrProfitDiffX, GlobalConstrProfitDiffY;

unfix Ax;
unfix Ay;
unfix mx ;
unfix my;

#########################
# Case 2: Estimate: sigma, gamme, A, m 
#
#########################

let {i in CITY} px[i] := AVGpx[i];
let {i in CITY} py[i] := AVGpy[i];

let Ax := 2 * max {i in CITY} AVGpx[i];
let Ay := 2 * max {i in CITY} AVGpy[i];
let mx := min {i in CITY} AVGpx[i];
let my := min {i in CITY} AVGpy[i];

# let gamma := 0.5;
# let sigma := 3;


let gamma := 0.25 + (2-0.25)/(25-1)*(k-1);
let sigma := 4 - (4-2)/(25-1)*(k-1);

let {i in CITY} Dx2[i] :=  n[i]*px[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));
let {i in CITY} Dy2[i] :=  n[i]*py[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));

solve PriceGameMLE;

printf "%u\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %15.6g\t  ", k, sigma, gamma, Ax, Ay, mx, my, px[1], py[1], px[2], py[2], px[3], py[3], px[4], py[4], Likelihood  > Case2.txt;

if match (solve_message, "LOCALLY OPTIMAL SOLUTION FOUND") > 0 then { 
			printf "%s\n", "Optimal solution found" > Case2.txt;
			}
		else if match (solve_message, "Iteration limit reached") > 0 then { 
			printf "%s\n", "Iteration limit reached" > Case2.txt;
			}
            else {
			printf "%s\n", "No solution" > Case2.txt;
            }


display px, py, gamma, sigma, Ax, Ay, mx, my;
display Dx1, Dx2, Dy2, Dy3;

display GlobalConstrProfitDiffX, GlobalConstrProfitDiffY;

#########################
# Case 3:  Estimate: sigma, gamma, Ax, Ay, mx, my;
#
#########################


let {i in CITY} px[i] := AVGpx[i];
let {i in CITY} py[i] := AVGpy[i];

let Ax := 2 * max {i in CITY} AVGpx[i];
let Ay := 2 * max {i in CITY} AVGpy[i];
let mx := min {i in CITY} AVGpx[i];
let my := min {i in CITY} AVGpy[i];

# let gamma := 0.5;
# let sigma := 3;


let gamma := 0.25 + (2-0.25)/(25-1)*(k-1);
let sigma := 4 - (4-2)/(25-1)*(k-1);

let {i in CITY} Dx2[i] :=  n[i]*px[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));
let {i in CITY} Dy2[i] :=  n[i]*py[i]^(-sigma)*(px[i]^(1-sigma)+py[i]^(1-sigma))^((1-gamma*sigma)/(gamma*(-1+sigma)));

drop AConstr;
drop mConstr;

solve PriceGameMLE;

printf "%u\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %6.4f\t %15.6g\t  ", k, sigma, gamma, Ax, Ay, mx, my, px[1], py[1], px[2], py[2], px[3], py[3], px[4], py[4], Likelihood  > Case3.txt;

if match (solve_message, "LOCALLY OPTIMAL SOLUTION FOUND") > 0 then { 
			printf "%s\n", "Optimal solution found" > Case3.txt;
			}
		else if match (solve_message, "Iteration limit reached") > 0 then { 
			printf "%s\n", "Iteration limit reached" > Case3.txt;
			}
            else {
			printf "%s\n", "No solution" > Case3.txt;
            }


display px, py, gamma, sigma, Ax, Ay, mx, my;
display Dx1, Dx2, Dy2, Dy3;

display GlobalConstrProfitDiffX, GlobalConstrProfitDiffY;

restore AConstr;
restore mConstr;

}


close Case1.txt;
close Case2.txt;
close Case3.txt;
